@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright 2020, 2021 Ramil Nugmanov <nougmanoff@protonmail.com>
+#  Copyright 2020-2022 Ramil Nugmanov <nougmanoff@protonmail.com>
 #  This file is part of chython.
 #
 #  chython is free software; you can redistribute it and/or modify
@@ -18,16 +18,18 @@
 #
 from fileinput import FileInput
 from io import StringIO, TextIOWrapper
+from itertools import repeat
 from pathlib import Path
 from traceback import format_exc
-from typing import Collection, Tuple, Optional
-from ._mdl import parse_error
+from typing import Optional, Iterable
 from .XYZrw import XYZ
+from ..exceptions import ParseError
 
 
 one_symbol_names = {'ALA', 'CYS', 'ASP', 'GLU', 'PHE', 'GLY', 'HIS', 'ILE', 'LYS', 'LEU', 'MET', 'ASN', 'PRO', 'GLN',
                     'ARG', 'SER', 'THR', 'VAL', 'TRP', 'TYR',  # Amino acids
                     'ASH',  # H-asparagine
+                    'HID', 'HIE', 'HIP',  # H-histidines
                     'DA', 'DC', 'DG', 'DT', 'DI',  # Deoxyribonucleotides
                     'A', 'C', 'G', 'U', 'I',  # Ribonucleotides
                     'IOD', 'K'}  # Elements
@@ -43,14 +45,16 @@ class PDBRead(XYZ):
     END or ENDMDL required in the end.
     """
     def __init__(self, file, ignore=False, element_name_priority=False, parse_as_single=False, atom_name_map=None,
-                 **kwargs):
+                 charge_map: Optional[Iterable[int]] = None, radical_map: Optional[Iterable[int]] = None, **kwargs):
         """
         :param ignore: Skip some checks of data or try to fix some errors.
         :param store_log: Store parser log if exists messages to `.meta` by key `ParserLog`.
         :param element_name_priority: For ligands use element symbol column value and ignore atom name column.
-        :param parse_as_single: Usable if all models in file is the same structure. 2d graph will be restored from first
-            model. Other models will be returned as conformers.
+        :param parse_as_single: Usable if all models in file is the same structure.
+            2d graph will be restored only from first model.
         :param atom_name_map: dictionary with atom names replacements. e.g.: {'Ow': 'O'}. Keys should be capitalized.
+        :param charge_map: iterable with total charges of each model in file.
+        :param radical_map: iterable with total radicals count of each model in file.
         """
         if isinstance(file, str):
             self._file = open(file)
@@ -70,6 +74,8 @@ class PDBRead(XYZ):
         self.__parse_as_single = parse_as_single
         self.__parsed_first = None
         self.__atom_name_map = atom_name_map or {}
+        self.__charge_map = charge_map
+        self.__radical_map = radical_map
         self._data = self.__reader()
 
     def __reader(self):
@@ -82,6 +88,8 @@ class PDBRead(XYZ):
             seekable = False
         ignore = self.__ignore
         failkey = False
+        charges = repeat(0) if self.__charge_map is None else iter(self.__charge_map)
+        radicals = repeat(0) if self.__radical_map is None else iter(self.__radical_map)
         atoms = []
 
         pos = 0 if seekable else None
@@ -89,6 +97,11 @@ class PDBRead(XYZ):
         for n, line in enumerate(self.__file):
             if failkey:
                 if line.startswith('END'):
+                    try:  # drop unused charges/radicals
+                        next(charges)
+                        next(radicals)
+                    except StopIteration:
+                        raise ValueError('charge_map or radical_map invalid')
                     failkey = False
                     if seekable:
                         pos = file.tell()
@@ -119,7 +132,7 @@ class PDBRead(XYZ):
                         self._info(f'Line [{n}]: consist errors in charge')
                         failkey = True
                         atoms = []
-                        yield parse_error(count, pos, self._format_log(), line)
+                        yield ParseError(count, pos, self._format_log(), line)
                 else:
                     charge = None
                 try:
@@ -128,12 +141,12 @@ class PDBRead(XYZ):
                     self._info(f'Line [{n}]: consist errors in coordinates definition')
                     failkey = True
                     atoms = []
-                    yield parse_error(count, pos, self._format_log(), line)
+                    yield ParseError(count, pos, self._format_log(), line)
                     continue
 
                 element = line[76:78].strip()
                 residue = line[17:20].strip()
-                atom_name = line[12:16].strip(' 0123456789')
+                atom_name = line[12:16].strip(' 0123456789-+')
                 if residue in one_symbol_names:  # bio-polymers and I
                     atom_name = atom_name[0]
                 elif residue in two_symbol_names:
@@ -160,45 +173,54 @@ class PDBRead(XYZ):
                     if not ignore:
                         failkey = True
                         atoms = []
-                        yield parse_error(count, pos, self._format_log(), line)
+                        yield ParseError(count, pos, self._format_log(), line)
                         continue
                 atoms.append((atom_name, charge, x, y, z, residue))
             elif line.startswith('END'):  # EOF or end of complex
                 if atoms:  # convert collected atoms
                     try:
-                        container = self._convert_molecule(atoms)
+                        container = self._convert_molecule(atoms, next(charges), next(radicals))
                     except ValueError:
                         self._info(f'Structure consist errors:\n{format_exc()}')
-                        yield parse_error(count, pos, self._format_log(), None)
+                        yield ParseError(count, pos, self._format_log(), None)
+                    except StopIteration:
+                        raise ValueError('charge_map or radical_map invalid')
                     else:
                         yield container
                     atoms = []
                 else:
+                    try:  # drop unused charges/radicals
+                        next(charges)
+                        next(radicals)
+                    except StopIteration:
+                        raise ValueError('charge_map or radical_map invalid')
                     self._info(f'Line [{n}]: END or ENDMDL before ATOM or HETATM')
-                    yield parse_error(count, pos, self._format_log(), line)
+                    yield ParseError(count, pos, self._format_log(), line)
                 count += 1
                 if seekable:
                     pos = file.tell()
         if atoms:  # ENDMDL or END not found
             self._info('PDB not finished')
-            yield parse_error(count, pos, self._format_log(), {})
+            yield ParseError(count, pos, self._format_log(), None)
 
-    def _convert_molecule(self, matrix: Collection[Tuple[str, Optional[int], float, float, float, str]]):
+    def _convert_molecule(self, matrix, charge=0, radical=0):
         if self.__parsed_first is None:
             mol = super()._convert_molecule([(e, c, x, y, z) for e, c, x, y, z, _ in matrix])
             mol.meta['RESIDUE'] = {n: x[-1] for n, x in zip(mol, matrix)}
             if self.__parse_as_single:
-                self.__parsed_first = [(n, a.atomic_symbol) for n, a in mol.atoms()]
+                self.__parsed_first = mol.copy()
             return mol
         else:
             if len(self.__parsed_first) != len(matrix):
                 raise ValueError('models not equal')
             c = {}
-            for (n, a), (e, _, x, y, z, _) in zip(self.__parsed_first, matrix):
-                if a != e:
+            for (n, a), (e, _, x, y, z, _) in zip(self.__parsed_first.atoms(), matrix):
+                if a.atomic_symbol != e:
                     raise ValueError('models or atom order not equal')
                 c[n] = (x, y, z)
-            return c
+            mol = self.__parsed_first.copy()
+            mol._conformers[0] = c
+            return mol
 
 
 __all__ = ['PDBRead']
